@@ -1,5 +1,6 @@
 import json
 import shlex
+import sys
 from asyncio import (CancelledError, create_subprocess_shell, create_task,
                      subprocess)
 from collections import namedtuple
@@ -11,7 +12,6 @@ import jedi
 import pyflakes.api
 import wordsegment
 from logzero import logger
-from xgboost import Booster, DMatrix
 
 from .completion_utilities import is_parameter_of_def
 from .config import config
@@ -20,12 +20,21 @@ from .jedi_preloader import preload_modules
 from .modeling.feature.feature_definition import tokenize
 from .online_feature_extractor import \
     OnlineFeatureExtractor  # 90ms, 10M memory
-from .project import save_state
+from .project import persistent_state, save_state
 from .pyflakes_reporter import PyflakesReporter
 from .utils import Timer, detect_doc_type, log_exception, nop
 from .websocket import register_handler
 from .word_completer import search_prefix
 
+# prevent pandas being imported by xgboost (save ~500ms)
+_pandas = sys.modules.get('pandas', None)
+if _pandas:
+    from xgboost.core import Booster, DMatrix
+else:
+    sys.modules['pandas'] = None
+    from xgboost.core import Booster, DMatrix
+    del sys.modules['pandas']
+
 DEBUG = False
 MODEL_NAME = 'v12.xgb'
 PredictionRow = namedtuple('PredictionRow', ('c', 't', 's', 'p'))
@@ -51,6 +60,8 @@ def get_relative_path(context):
 async def run_pylint(context, send):
     if not config['linter']['pylint']:
         return
+    if context.path.suffix != '.py':
+        return
     try:
         with Timer('Linting'):
             absolute_path = context.path.absolute()
@@ -76,6 +87,8 @@ async def run_pylint(context, send):
 async def run_yapf(context):
     if not config['formatter']['yapf']:
         return
+    if context.path.suffix != '.py':
+        return
     with log_exception():
         with Timer('YAPF'):
             absolute_path = context.path.absolute()
@@ -94,6 +107,8 @@ async def run_yapf(context):
 async def run_isort(context):
     if not config['formatter']['isort']:
         return
+    if context.path.suffix != '.py':
+        return
     with log_exception():
         with Timer('Sorting'):
             absolute_path = context.path.absolute()
@@ -123,6 +138,8 @@ async def run_spell_checker(context, send):
 async def run_pyflakes(context, send):
     if not config['linter']['pyflakes']:
         return
+    if context.path.suffix != '.py':
+        return
     reporter = context.pyflakes_reporter
     reporter.clear()
     pyflakes.api.check(context.content, '', reporter)
@@ -169,6 +186,8 @@ async def connected(msg, send, context):
     context.linter_task = create_task(nop())
     # open file
     context.path = Path(context.shared.project_root, *msg['filePath'])
+    file_state = persistent_state.get_file_state(context.path)
+    context.pos = file_state.get('pos', (0, 0))
     context.is_python = context.path.suffix in ('.py', '.pyx')
     context.pyflakes_reporter = PyflakesReporter()
     with open(context.path) as f:
@@ -184,7 +203,8 @@ async def connected(msg, send, context):
     # somehow risky, but it should not wait until the extractor ready
     await send('FileOpened', {
         'mtime': context.path.stat().st_mtime,
-        'content': content
+        'content': content,
+        **file_state,
     })
     # update opened files
     opened_files = context.shared.project_config['openedFiles']
@@ -207,6 +227,7 @@ async def warm_up(context, send):
 @handles('_disconnected')
 async def disconnected(context):
     context.linter_task.cancel()
+    persistent_state.set_file_state(context.path, {'pos': context.pos})
 
 
 @handles('Close')
@@ -216,9 +237,15 @@ async def close(msg, send, context):
     """
     opened_files = context.shared.project_config['openedFiles']
     opened_files.remove(get_relative_path(context))
+    context.pos = msg['pos']
     save_state(context)
 
 
+@handles('Blur')
+async def blur(msg, send, context):
+    context.pos = msg['pos']
+
+
 @handles('Reload')
 async def reload(msg, send, context):
     with open(context.path) as f:
@@ -365,6 +392,7 @@ async def predict(msg, send, context):
             'offset': offset,
             'result': result,
         })
+        context.pos = (line_number, ch)
     except Exception as e:
         logger.exception(e)
         await send('RequestFullSync', None)
