@@ -1,18 +1,27 @@
 import json
+import os
 import sqlite3
 from functools import partial
 from importlib import resources
 from pathlib import Path
 
-from git import InvalidGitRepositoryError, Repo
 from logzero import logger
 
 from .config import config, set_config
-from .file_finder import find_in_directory, replace_all_in_directory
+from .file_finder import (find_in_directory, get_pathspec,
+                          replace_all_in_directory)
 from .spell_checker import SpellChecker
 from .utils import config_directory, merge_dict
 from .websocket import register_handler
 
+try:
+    from git import InvalidGitRepositoryError, Repo
+    git_available = True
+except ImportError:
+    git_available = False
+    logger.warning('Git is not available, please install git to enable version control integration.')
+
+
 
 class PersistentConfig:
     def __init__(self):
@@ -23,6 +32,8 @@ class PersistentConfig:
                                   isolation_level=None)
         self.db.execute(
             'CREATE TABLE IF NOT EXISTS c(k TEXT PRIMARY KEY, v TEXT)')
+        self.db.execute(
+            'CREATE TABLE IF NOT EXISTS f(k TEXT PRIMARY KEY, v TEXT)')
 
     def __setitem__(self, key, value):
         key = str(key)
@@ -47,6 +58,19 @@ class PersistentConfig:
     def close(self):
         self.db.close()
 
+    def set_file_state(self, key, value):
+        key = str(key)
+        j = json.dumps(value)
+        self.db.execute('INSERT OR REPLACE INTO f(k, v) VALUES (?, ?)',
+                        (key, j))
+
+    def get_file_state(self, key):
+        key = str(key)
+        result = self.db.execute('SELECT v FROM f WHERE k=?',
+                                 (key, )).fetchall()
+        result = json.loads(result[0][0]) if result else {}
+        return result
+
 
 handles = partial(register_handler, 'project')
 persistent_state = PersistentConfig()
@@ -79,6 +103,9 @@ async def open_project(msg, send, context):
 
 @handles('RequestGitStatusUpdate')
 async def request_git_status_update(msg, send, context):
+    if not git_available:
+        logger.debug('Git unavailable')
+        return 
     sc = context.shared
     if not sc.repo:
         try:
@@ -119,3 +146,31 @@ def save_state(context):
 
 handles('FindInDirectory')(find_in_directory)
 handles('ReplaceAllInDirectory')(replace_all_in_directory)
+
+
+@handles('FindFileByName')
+async def find_file_by_name(msg, send, context):
+    sep = os.sep
+    project_root = context.shared.project_root
+    pathspec = get_pathspec(project_root)
+    limit = msg['limit']
+    keywords = [i.lower() for i in msg['keywords'].split()]
+    result = []
+    for root, _, files in os.walk(project_root):
+        if '__pycache__' in root:
+            continue
+        for file in files:
+            file_lower = file.lower()
+            for keyword in keywords:
+                if keyword not in file_lower:
+                    break
+            else:
+                path = Path(root) / file
+                relative_path = path.relative_to(project_root)
+                if pathspec.match_file(str(relative_path)):
+                    continue
+                result.append(str(Path(root, file).relative_to(project_root)))
+                if len(result) >= limit:
+                    await send('FileFound', result)
+                    return
+    await send('FileFound', result)
